import { Injectable } from '@angular/core';
import { assign, cloneDeep } from 'lodash-es';
import { FuseMockApiService, FuseMockApiUtils } from '@fuse/lib/mock-api';
import { partnerMainoffices as partnerMainofficesData } from './data';

@Injectable({
  providedIn: 'root',
})
export class MemberPartnerMainofficeMockApi {
  private _partnerMainoffices: any[] = partnerMainofficesData;

  /**
   * Constructor
   */
  constructor(private _fuseMockApiService: FuseMockApiService) {
    // Register Mock API handlers
    this.registerHandlers();
  }

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Register Mock API handlers
   */
  registerHandlers(): void {
    // -----------------------------------------------------------------------------------------------------
    // @ PartnerMainoffices - GET
    // -----------------------------------------------------------------------------------------------------
    this._fuseMockApiService
      .onGet('api/apps/member/partner-mainoffice/partner-mainoffices', 300)
      .reply(({ request }) => {
        // Get available queries
        const search = request.params.get('search');
        const sort = request.params.get('sort') || 'name';
        const order = request.params.get('order') || 'asc';
        const page = parseInt(request.params.get('page') ?? '1', 10);
        const size = parseInt(request.params.get('size') ?? '10', 10);

        // Clone the partnerMainoffices
        let partnerMainoffices: any[] | null = cloneDeep(
          this._partnerMainoffices
        );

        // Sort the partnerMainoffices
        if (sort === 'sku' || sort === 'name' || sort === 'active') {
          partnerMainoffices.sort((a, b) => {
            const fieldA = a[sort].toString().toUpperCase();
            const fieldB = b[sort].toString().toUpperCase();
            return order === 'asc'
              ? fieldA.localeCompare(fieldB)
              : fieldB.localeCompare(fieldA);
          });
        } else {
          partnerMainoffices.sort((a, b) =>
            order === 'asc' ? a[sort] - b[sort] : b[sort] - a[sort]
          );
        }

        // If search exists...
        if (search) {
          // Filter the partnerMainoffices
          partnerMainoffices = partnerMainoffices.filter(
            (contact: any) =>
              contact.name &&
              contact.name.toLowerCase().includes(search.toLowerCase())
          );
        }

        // Paginate - Start
        const partnerMainofficesLength = partnerMainoffices.length;

        // Calculate pagination details
        const begin = page * size;
        const end = Math.min(size * (page + 1), partnerMainofficesLength);
        const lastPage = Math.max(
          Math.ceil(partnerMainofficesLength / size),
          1
        );

        // Prepare the pagination object
        let pagination = {};

        // If the requested page number is bigger than
        // the last possible page number, return null for
        // partnerMainoffices but also send the last possible page so
        // the app can navigate to there
        if (page > lastPage) {
          partnerMainoffices = null;
          pagination = {
            lastPage,
          };
        } else {
          // Paginate the results by size
          partnerMainoffices = partnerMainoffices.slice(begin, end);

          // Prepare the pagination mock-api
          pagination = {
            length: partnerMainofficesLength,
            size: size,
            page: page,
            lastPage: lastPage,
            startIndex: begin,
            endIndex: end - 1,
          };
        }

        // Return the response
        return [
          200,
          {
            partnerMainoffices,
            pagination,
          },
        ];
      });

    // -----------------------------------------------------------------------------------------------------
    // @ PartnerMainoffice - GET
    // -----------------------------------------------------------------------------------------------------
    this._fuseMockApiService
      .onGet('api/apps/member/partne-mainoffice/partner-mainoffice')
      .reply(({ request }) => {
        // Get the id from the params
        const id = request.params.get('id');

        // Clone the partnerMainoffices
        const partnerMainoffices = cloneDeep(this._partnerMainoffices);

        // Find the partnerMainoffice
        const partnerMainoffice = partnerMainoffices.find(
          (item: any) => item.id === id
        );

        // Return the response
        return [200, partnerMainoffice];
      });

    // -----------------------------------------------------------------------------------------------------
    // @ PartnerMainoffice - POST
    // -----------------------------------------------------------------------------------------------------
    this._fuseMockApiService
      .onPost('api/apps/member/partner-mainoffice/partner-mainoffice')
      .reply(() => {
        // Generate a new partnerMainoffice
        const newPartnerMainoffice = {
          id: FuseMockApiUtils.guid(),
          category: '',
          name: 'A New User',
          description: '',
          tags: [],
          sku: '',
          barcode: '',
          brand: '',
          vendor: '',
          stock: '',
          reserved: '',
          cost: '',
          basePrice: '',
          taxPercent: '',
          price: '',
          weight: '',
          thumbnail: '',
          images: [],
          active: false,
        };

        // Unshift the new partnerMainoffice
        this._partnerMainoffices.unshift(newPartnerMainoffice);

        // Return the response
        return [200, newPartnerMainoffice];
      });

    // -----------------------------------------------------------------------------------------------------
    // @ PartnerMainoffice - PATCH
    // -----------------------------------------------------------------------------------------------------
    this._fuseMockApiService
      .onPatch('api/apps/member/partner-mainoffice/partner-mainoffice')
      .reply(({ request }) => {
        // Get the id and partnerMainoffice
        const id = request.body.id;
        const partnerMainoffice = cloneDeep(request.body.partnerMainoffice);

        // Prepare the updated partnerMainoffice
        let updatedPartnerMainoffice = null;

        // Find the partnerMainoffice and update it
        this._partnerMainoffices.forEach((item, index, partnerMainoffices) => {
          if (item.id === id) {
            // Update the partnerMainoffice
            partnerMainoffices[index] = assign(
              {},
              partnerMainoffices[index],
              partnerMainoffice
            );

            // Store the updated partnerMainoffice
            updatedPartnerMainoffice = partnerMainoffices[index];
          }
        });

        // Return the response
        return [200, updatedPartnerMainoffice];
      });

    // -----------------------------------------------------------------------------------------------------
    // @ PartnerMainoffice - DELETE
    // -----------------------------------------------------------------------------------------------------
    this._fuseMockApiService
      .onDelete('api/apps/member/partner-mainoffice/partner-mainoffice')
      .reply(({ request }) => {
        // Get the id
        const id = request.params.get('id');

        // Find the partnerMainoffice and delete it
        this._partnerMainoffices.forEach((item, index) => {
          if (item.id === id) {
            this._partnerMainoffices.splice(index, 1);
          }
        });

        // Return the response
        return [200, true];
      });
  }
}
