import { Injectable } from '@angular/core';
import { assign, cloneDeep } from 'lodash-es';
import { FuseMockApiService, FuseMockApiUtils } from '@fuse/lib/mock-api';
import { sessioninAdmins as sessioninAdminsData } from './data';

@Injectable({
  providedIn: 'root',
})
export class ReportSessioninAdminMockApi {
  private _sessioninAdmins: any[] = sessioninAdminsData;

  /**
   * Constructor
   */
  constructor(private _fuseMockApiService: FuseMockApiService) {
    // Register Mock API handlers
    this.registerHandlers();
  }

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Register Mock API handlers
   */
  registerHandlers(): void {
    // -----------------------------------------------------------------------------------------------------
    // @ SessioninAdmins - GET
    // -----------------------------------------------------------------------------------------------------
    this._fuseMockApiService
      .onGet('api/apps/report/sessionin-admin/sessionin-admins', 300)
      .reply(({ request }) => {
        // Get available queries
        const search = request.params.get('search');
        const sort = request.params.get('sort') || 'name';
        const order = request.params.get('order') || 'asc';
        const page = parseInt(request.params.get('page') ?? '1', 10);
        const size = parseInt(request.params.get('size') ?? '10', 10);

        // Clone the sessioninAdmins
        let sessioninAdmins: any[] | null = cloneDeep(this._sessioninAdmins);

        // Sort the sessioninAdmins
        if (sort === 'sku' || sort === 'name' || sort === 'active') {
          sessioninAdmins.sort((a, b) => {
            const fieldA = a[sort].toString().toUpperCase();
            const fieldB = b[sort].toString().toUpperCase();
            return order === 'asc'
              ? fieldA.localeCompare(fieldB)
              : fieldB.localeCompare(fieldA);
          });
        } else {
          sessioninAdmins.sort((a, b) =>
            order === 'asc' ? a[sort] - b[sort] : b[sort] - a[sort]
          );
        }

        // If search exists...
        if (search) {
          // Filter the sessioninAdmins
          sessioninAdmins = sessioninAdmins.filter(
            (contact: any) =>
              contact.name &&
              contact.name.toLowerCase().includes(search.toLowerCase())
          );
        }

        // Paginate - Start
        const sessioninAdminsLength = sessioninAdmins.length;

        // Calculate pagination details
        const begin = page * size;
        const end = Math.min(size * (page + 1), sessioninAdminsLength);
        const lastPage = Math.max(Math.ceil(sessioninAdminsLength / size), 1);

        // Prepare the pagination object
        let pagination = {};

        // If the requested page number is bigger than
        // the last possible page number, return null for
        // sessioninAdmins but also send the last possible page so
        // the app can navigate to there
        if (page > lastPage) {
          sessioninAdmins = null;
          pagination = {
            lastPage,
          };
        } else {
          // Paginate the results by size
          sessioninAdmins = sessioninAdmins.slice(begin, end);

          // Prepare the pagination mock-api
          pagination = {
            length: sessioninAdminsLength,
            size: size,
            page: page,
            lastPage: lastPage,
            startIndex: begin,
            endIndex: end - 1,
          };
        }

        // Return the response
        return [
          200,
          {
            sessioninAdmins,
            pagination,
          },
        ];
      });

    // -----------------------------------------------------------------------------------------------------
    // @ SessioninAdmin - GET
    // -----------------------------------------------------------------------------------------------------
    this._fuseMockApiService
      .onGet('api/apps/report/sessionin-admin/sessionin-admin')
      .reply(({ request }) => {
        // Get the id from the params
        const id = request.params.get('id');

        // Clone the sessioninAdmins
        const sessioninAdmins = cloneDeep(this._sessioninAdmins);

        // Find the sessioninAdmin
        const sessioninAdmin = sessioninAdmins.find(
          (item: any) => item.id === id
        );

        // Return the response
        return [200, sessioninAdmin];
      });

    // -----------------------------------------------------------------------------------------------------
    // @ SessioninAdmin - POST
    // -----------------------------------------------------------------------------------------------------
    this._fuseMockApiService
      .onPost('api/apps/report/sessionin-admin/sessionin-admin')
      .reply(() => {
        // Generate a new sessioninAdmin
        const newSessioninAdmin = {
          id: FuseMockApiUtils.guid(),
          category: '',
          name: 'A New User',
          description: '',
          tags: [],
          sku: '',
          barcode: '',
          brand: '',
          vendor: '',
          stock: '',
          reserved: '',
          cost: '',
          basePrice: '',
          taxPercent: '',
          price: '',
          weight: '',
          thumbnail: '',
          images: [],
          active: false,
        };

        // Unshift the new sessioninAdmin
        this._sessioninAdmins.unshift(newSessioninAdmin);

        // Return the response
        return [200, newSessioninAdmin];
      });

    // -----------------------------------------------------------------------------------------------------
    // @ SessioninAdmin - PATCH
    // -----------------------------------------------------------------------------------------------------
    this._fuseMockApiService
      .onPatch('api/apps/report/sessionin-admin/sessionin-admin')
      .reply(({ request }) => {
        // Get the id and sessioninAdmin
        const id = request.body.id;
        const sessioninAdmin = cloneDeep(request.body.sessioninAdmin);

        // Prepare the updated sessioninAdmin
        let updatedSessioninAdmin = null;

        // Find the sessioninAdmin and update it
        this._sessioninAdmins.forEach((item, index, sessioninAdmins) => {
          if (item.id === id) {
            // Update the sessioninAdmin
            sessioninAdmins[index] = assign(
              {},
              sessioninAdmins[index],
              sessioninAdmin
            );

            // Store the updated sessioninAdmin
            updatedSessioninAdmin = sessioninAdmins[index];
          }
        });

        // Return the response
        return [200, updatedSessioninAdmin];
      });

    // -----------------------------------------------------------------------------------------------------
    // @ SessioninAdmin - DELETE
    // -----------------------------------------------------------------------------------------------------
    this._fuseMockApiService
      .onDelete('api/apps/report/sessionin-admin/sessionin-admin')
      .reply(({ request }) => {
        // Get the id
        const id = request.params.get('id');

        // Find the sessioninAdmin and delete it
        this._sessioninAdmins.forEach((item, index) => {
          if (item.id === id) {
            this._sessioninAdmins.splice(index, 1);
          }
        });

        // Return the response
        return [200, true];
      });
  }
}
