import { Injectable } from '@angular/core';
import { assign, cloneDeep } from 'lodash-es';
import { FuseMockApiService, FuseMockApiUtils } from '@fuse/lib/mock-api';
import { sessioninInfos as sessioninInfosData } from './data';

@Injectable({
  providedIn: 'root',
})
export class ReportSessioninInfoMockApi {
  private _sessioninInfos: any[] = sessioninInfosData;

  /**
   * Constructor
   */
  constructor(private _fuseMockApiService: FuseMockApiService) {
    // Register Mock API handlers
    this.registerHandlers();
  }

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Register Mock API handlers
   */
  registerHandlers(): void {
    // -----------------------------------------------------------------------------------------------------
    // @ SessioninInfos - GET
    // -----------------------------------------------------------------------------------------------------
    this._fuseMockApiService
      .onGet('api/apps/report/sessionin-info/sessionin-infos', 300)
      .reply(({ request }) => {
        // Get available queries
        const search = request.params.get('search');
        const sort = request.params.get('sort') || 'name';
        const order = request.params.get('order') || 'asc';
        const page = parseInt(request.params.get('page') ?? '1', 10);
        const size = parseInt(request.params.get('size') ?? '10', 10);

        // Clone the sessioninInfos
        let sessioninInfos: any[] | null = cloneDeep(this._sessioninInfos);

        // Sort the sessioninInfos
        if (sort === 'sku' || sort === 'name' || sort === 'active') {
          sessioninInfos.sort((a, b) => {
            const fieldA = a[sort].toString().toUpperCase();
            const fieldB = b[sort].toString().toUpperCase();
            return order === 'asc'
              ? fieldA.localeCompare(fieldB)
              : fieldB.localeCompare(fieldA);
          });
        } else {
          sessioninInfos.sort((a, b) =>
            order === 'asc' ? a[sort] - b[sort] : b[sort] - a[sort]
          );
        }

        // If search exists...
        if (search) {
          // Filter the sessioninInfos
          sessioninInfos = sessioninInfos.filter(
            (contact: any) =>
              contact.name &&
              contact.name.toLowerCase().includes(search.toLowerCase())
          );
        }

        // Paginate - Start
        const sessioninInfosLength = sessioninInfos.length;

        // Calculate pagination details
        const begin = page * size;
        const end = Math.min(size * (page + 1), sessioninInfosLength);
        const lastPage = Math.max(Math.ceil(sessioninInfosLength / size), 1);

        // Prepare the pagination object
        let pagination = {};

        // If the requested page number is bigger than
        // the last possible page number, return null for
        // sessioninInfos but also send the last possible page so
        // the app can navigate to there
        if (page > lastPage) {
          sessioninInfos = null;
          pagination = {
            lastPage,
          };
        } else {
          // Paginate the results by size
          sessioninInfos = sessioninInfos.slice(begin, end);

          // Prepare the pagination mock-api
          pagination = {
            length: sessioninInfosLength,
            size: size,
            page: page,
            lastPage: lastPage,
            startIndex: begin,
            endIndex: end - 1,
          };
        }

        // Return the response
        return [
          200,
          {
            sessioninInfos,
            pagination,
          },
        ];
      });

    // -----------------------------------------------------------------------------------------------------
    // @ SessioninInfo - GET
    // -----------------------------------------------------------------------------------------------------
    this._fuseMockApiService
      .onGet('api/apps/report/sessionin-info/sessionin-info')
      .reply(({ request }) => {
        // Get the id from the params
        const id = request.params.get('id');

        // Clone the sessioninInfos
        const sessioninInfos = cloneDeep(this._sessioninInfos);

        // Find the sessioninInfo
        const sessioninInfo = sessioninInfos.find(
          (item: any) => item.id === id
        );

        // Return the response
        return [200, sessioninInfo];
      });

    // -----------------------------------------------------------------------------------------------------
    // @ SessioninInfo - POST
    // -----------------------------------------------------------------------------------------------------
    this._fuseMockApiService
      .onPost('api/apps/report/sessionin-info/sessionin-info')
      .reply(() => {
        // Generate a new sessioninInfo
        const newSessioninInfo = {
          id: FuseMockApiUtils.guid(),
          category: '',
          name: 'A New User',
          description: '',
          tags: [],
          sku: '',
          barcode: '',
          brand: '',
          vendor: '',
          stock: '',
          reserved: '',
          cost: '',
          basePrice: '',
          taxPercent: '',
          price: '',
          weight: '',
          thumbnail: '',
          images: [],
          active: false,
        };

        // Unshift the new sessioninInfo
        this._sessioninInfos.unshift(newSessioninInfo);

        // Return the response
        return [200, newSessioninInfo];
      });

    // -----------------------------------------------------------------------------------------------------
    // @ SessioninInfo - PATCH
    // -----------------------------------------------------------------------------------------------------
    this._fuseMockApiService
      .onPatch('api/apps/report/sessionin-info/sessionin-info')
      .reply(({ request }) => {
        // Get the id and sessioninInfo
        const id = request.body.id;
        const sessioninInfo = cloneDeep(request.body.sessioninInfo);

        // Prepare the updated sessioninInfo
        let updatedSessioninInfo = null;

        // Find the sessioninInfo and update it
        this._sessioninInfos.forEach((item, index, sessioninInfos) => {
          if (item.id === id) {
            // Update the sessioninInfo
            sessioninInfos[index] = assign(
              {},
              sessioninInfos[index],
              sessioninInfo
            );

            // Store the updated sessioninInfo
            updatedSessioninInfo = sessioninInfos[index];
          }
        });

        // Return the response
        return [200, updatedSessioninInfo];
      });

    // -----------------------------------------------------------------------------------------------------
    // @ SessioninInfo - DELETE
    // -----------------------------------------------------------------------------------------------------
    this._fuseMockApiService
      .onDelete('api/apps/report/sessionin-info/sessionin-info')
      .reply(({ request }) => {
        // Get the id
        const id = request.params.get('id');

        // Find the sessioninInfo and delete it
        this._sessioninInfos.forEach((item, index) => {
          if (item.id === id) {
            this._sessioninInfos.splice(index, 1);
          }
        });

        // Return the response
        return [200, true];
      });
  }
}
