import { Injectable } from '@angular/core';
import { assign, cloneDeep } from 'lodash-es';
import { FuseMockApiService, FuseMockApiUtils } from '@fuse/lib/mock-api';
import { sessioninOverlaps as sessioninOverlapsData } from './data';

@Injectable({
  providedIn: 'root',
})
export class ReportSessioninOverlapMockApi {
  private _sessioninOverlaps: any[] = sessioninOverlapsData;

  /**
   * Constructor
   */
  constructor(private _fuseMockApiService: FuseMockApiService) {
    // Register Mock API handlers
    this.registerHandlers();
  }

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Register Mock API handlers
   */
  registerHandlers(): void {
    // -----------------------------------------------------------------------------------------------------
    // @ SessioninOverlaps - GET
    // -----------------------------------------------------------------------------------------------------
    this._fuseMockApiService
      .onGet('api/apps/report/sessionin-overlap/sessionin-overlaps', 300)
      .reply(({ request }) => {
        // Get available queries
        const search = request.params.get('search');
        const sort = request.params.get('sort') || 'name';
        const order = request.params.get('order') || 'asc';
        const page = parseInt(request.params.get('page') ?? '1', 10);
        const size = parseInt(request.params.get('size') ?? '10', 10);

        // Clone the sessioninoverlaps
        let sessioninOverlaps: any[] | null = cloneDeep(
          this._sessioninOverlaps
        );

        // Sort the sessioninOverlaps
        if (sort === 'sku' || sort === 'name' || sort === 'active') {
          sessioninOverlaps.sort((a, b) => {
            const fieldA = a[sort].toString().toUpperCase();
            const fieldB = b[sort].toString().toUpperCase();
            return order === 'asc'
              ? fieldA.localeCompare(fieldB)
              : fieldB.localeCompare(fieldA);
          });
        } else {
          sessioninOverlaps.sort((a, b) =>
            order === 'asc' ? a[sort] - b[sort] : b[sort] - a[sort]
          );
        }

        // If search exists...
        if (search) {
          // Filter the sessioninOverlaps
          sessioninOverlaps = sessioninOverlaps.filter(
            (contact: any) =>
              contact.name &&
              contact.name.toLowerCase().includes(search.toLowerCase())
          );
        }

        // Paginate - Start
        const sessioninOverlapsLength = sessioninOverlaps.length;

        // Calculate pagination details
        const begin = page * size;
        const end = Math.min(size * (page + 1), sessioninOverlapsLength);
        const lastPage = Math.max(Math.ceil(sessioninOverlapsLength / size), 1);

        // Prepare the pagination object
        let pagination = {};

        // If the requested page number is bigger than
        // the last possible page number, return null for
        // sessioninOverlaps but also send the last possible page so
        // the app can navigate to there
        if (page > lastPage) {
          sessioninOverlaps = null;
          pagination = {
            lastPage,
          };
        } else {
          // Paginate the results by size
          sessioninOverlaps = sessioninOverlaps.slice(begin, end);

          // Prepare the pagination mock-api
          pagination = {
            length: sessioninOverlapsLength,
            size: size,
            page: page,
            lastPage: lastPage,
            startIndex: begin,
            endIndex: end - 1,
          };
        }

        // Return the response
        return [
          200,
          {
            sessioninOverlaps,
            pagination,
          },
        ];
      });

    // -----------------------------------------------------------------------------------------------------
    // @ SessioninOverlap - GET
    // -----------------------------------------------------------------------------------------------------
    this._fuseMockApiService
      .onGet('api/apps/report/sessionin-overlap/sessionin-overlap')
      .reply(({ request }) => {
        // Get the id from the params
        const id = request.params.get('id');

        // Clone the sessioninOverlaps
        const sessioninOverlaps = cloneDeep(this._sessioninOverlaps);

        // Find the sessioninOverlap
        const sessioninOverlap = sessioninOverlaps.find(
          (item: any) => item.id === id
        );

        // Return the response
        return [200, sessioninOverlap];
      });

    // -----------------------------------------------------------------------------------------------------
    // @ SessioninOverlap - POST
    // -----------------------------------------------------------------------------------------------------
    this._fuseMockApiService
      .onPost('api/apps/report/sessionin-overlap/sessionin-overlap')
      .reply(() => {
        // Generate a new sessioninOverlap
        const newSessioninOverlap = {
          id: FuseMockApiUtils.guid(),
          category: '',
          name: 'A New User',
          description: '',
          tags: [],
          sku: '',
          barcode: '',
          brand: '',
          vendor: '',
          stock: '',
          reserved: '',
          cost: '',
          basePrice: '',
          taxPercent: '',
          price: '',
          weight: '',
          thumbnail: '',
          images: [],
          active: false,
        };

        // Unshift the new sessioninOverlap
        this._sessioninOverlaps.unshift(newSessioninOverlap);

        // Return the response
        return [200, newSessioninOverlap];
      });

    // -----------------------------------------------------------------------------------------------------
    // @ SessioninOverlap - PATCH
    // -----------------------------------------------------------------------------------------------------
    this._fuseMockApiService
      .onPatch('api/apps/report/sessionin-overlap/sessionin-overlap')
      .reply(({ request }) => {
        // Get the id and sessioninOverlap
        const id = request.body.id;
        const sessioninOverlap = cloneDeep(request.body.sessioninOverlap);

        // Prepare the updated sessioninOverlap
        let updatedSessioninOverlap = null;

        // Find the sessioninOverlap and update it
        this._sessioninOverlaps.forEach((item, index, sessioninOverlaps) => {
          if (item.id === id) {
            // Update the sessioninOverlap
            sessioninOverlaps[index] = assign(
              {},
              sessioninOverlaps[index],
              sessioninOverlap
            );

            // Store the updated sessioninOverlap
            updatedSessioninOverlap = sessioninOverlaps[index];
          }
        });

        // Return the response
        return [200, updatedSessioninOverlap];
      });

    // -----------------------------------------------------------------------------------------------------
    // @ SessioninOverlap - DELETE
    // -----------------------------------------------------------------------------------------------------
    this._fuseMockApiService
      .onDelete('api/apps/report/sessionin-overlap/sessionin-overlap')
      .reply(({ request }) => {
        // Get the id
        const id = request.params.get('id');

        // Find the sessioninOverlap and delete it
        this._sessioninOverlaps.forEach((item, index) => {
          if (item.id === id) {
            this._sessioninOverlaps.splice(index, 1);
          }
        });

        // Return the response
        return [200, true];
      });
  }
}
