import { Injectable } from '@angular/core';
import { assign, cloneDeep } from 'lodash-es';
import { FuseMockApiService, FuseMockApiUtils } from '@fuse/lib/mock-api';
import { statisticss as statisticssData } from './data';

@Injectable({
  providedIn: 'root',
})
export class ReportStatisticsMockApi {
  private _statisticss: any[] = statisticssData;

  /**
   * Constructor
   */
  constructor(private _fuseMockApiService: FuseMockApiService) {
    // Register Mock API handlers
    this.registerHandlers();
  }

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Register Mock API handlers
   */
  registerHandlers(): void {
    // -----------------------------------------------------------------------------------------------------
    // @ Statisticss - GET
    // -----------------------------------------------------------------------------------------------------
    this._fuseMockApiService
      .onGet('api/apps/report/statistics/statisticss', 300)
      .reply(({ request }) => {
        // Get available queries
        const search = request.params.get('search');
        const sort = request.params.get('sort') || 'name';
        const order = request.params.get('order') || 'asc';
        const page = parseInt(request.params.get('page') ?? '1', 10);
        const size = parseInt(request.params.get('size') ?? '10', 10);

        // Clone the statisticss
        let statisticss: any[] | null = cloneDeep(this._statisticss);

        // Sort the statisticss
        if (sort === 'sku' || sort === 'name' || sort === 'active') {
          statisticss.sort((a, b) => {
            const fieldA = a[sort].toString().toUpperCase();
            const fieldB = b[sort].toString().toUpperCase();
            return order === 'asc'
              ? fieldA.localeCompare(fieldB)
              : fieldB.localeCompare(fieldA);
          });
        } else {
          statisticss.sort((a, b) =>
            order === 'asc' ? a[sort] - b[sort] : b[sort] - a[sort]
          );
        }

        // If search exists...
        if (search) {
          // Filter the statisticss
          statisticss = statisticss.filter(
            (contact: any) =>
              contact.name &&
              contact.name.toLowerCase().includes(search.toLowerCase())
          );
        }

        // Paginate - Start
        const statisticssLength = statisticss.length;

        // Calculate pagination details
        const begin = page * size;
        const end = Math.min(size * (page + 1), statisticssLength);
        const lastPage = Math.max(Math.ceil(statisticssLength / size), 1);

        // Prepare the pagination object
        let pagination = {};

        // If the requested page number is bigger than
        // the last possible page number, return null for
        // statisticss but also send the last possible page so
        // the app can navigate to there
        if (page > lastPage) {
          statisticss = null;
          pagination = {
            lastPage,
          };
        } else {
          // Paginate the results by size
          statisticss = statisticss.slice(begin, end);

          // Prepare the pagination mock-api
          pagination = {
            length: statisticssLength,
            size: size,
            page: page,
            lastPage: lastPage,
            startIndex: begin,
            endIndex: end - 1,
          };
        }

        // Return the response
        return [
          200,
          {
            statisticss,
            pagination,
          },
        ];
      });

    // -----------------------------------------------------------------------------------------------------
    // @ Statistics - GET
    // -----------------------------------------------------------------------------------------------------
    this._fuseMockApiService
      .onGet('api/apps/report/statistics/statistics')
      .reply(({ request }) => {
        // Get the id from the params
        const id = request.params.get('id');

        // Clone the statisticss
        const statisticss = cloneDeep(this._statisticss);

        // Find the statistics
        const statistics = statisticss.find((item: any) => item.id === id);

        // Return the response
        return [200, statistics];
      });

    // -----------------------------------------------------------------------------------------------------
    // @ Statistics - POST
    // -----------------------------------------------------------------------------------------------------
    this._fuseMockApiService
      .onPost('api/apps/report/statistics/statistics')
      .reply(() => {
        // Generate a new statistics
        const newStatistics = {
          id: FuseMockApiUtils.guid(),
          category: '',
          name: 'A New User',
          description: '',
          tags: [],
          sku: '',
          barcode: '',
          brand: '',
          vendor: '',
          stock: '',
          reserved: '',
          cost: '',
          basePrice: '',
          taxPercent: '',
          price: '',
          weight: '',
          thumbnail: '',
          images: [],
          active: false,
        };

        // Unshift the new statistics
        this._statisticss.unshift(newStatistics);

        // Return the response
        return [200, newStatistics];
      });

    // -----------------------------------------------------------------------------------------------------
    // @ Statistics - PATCH
    // -----------------------------------------------------------------------------------------------------
    this._fuseMockApiService
      .onPatch('api/apps/report/statistics/statistics')
      .reply(({ request }) => {
        // Get the id and statistics
        const id = request.body.id;
        const statistics = cloneDeep(request.body.statistics);

        // Prepare the updated statistics
        let updatedStatistics = null;

        // Find the statistics and update it
        this._statisticss.forEach((item, index, statisticss) => {
          if (item.id === id) {
            // Update the statistics
            statisticss[index] = assign({}, statisticss[index], statistics);

            // Store the updated Statistics
            updatedStatistics = statisticss[index];
          }
        });

        // Return the response
        return [200, updatedStatistics];
      });

    // -----------------------------------------------------------------------------------------------------
    // @ Statistics - DELETE
    // -----------------------------------------------------------------------------------------------------
    this._fuseMockApiService
      .onDelete('api/apps/report/statistics/statistics')
      .reply(({ request }) => {
        // Get the id
        const id = request.params.get('id');

        // Find the statistics and delete it
        this._statisticss.forEach((item, index) => {
          if (item.id === id) {
            this._statisticss.splice(index, 1);
          }
        });

        // Return the response
        return [200, true];
      });
  }
}
