import { Injectable } from '@angular/core';
import { HttpClient } from '@angular/common/http';
import {
  BehaviorSubject,
  filter,
  map,
  Observable,
  of,
  switchMap,
  take,
  tap,
  throwError,
} from 'rxjs';

import { Statistics } from '../models/statistics';
import { StatisticsPagination } from '../models/statistics-pagination';

@Injectable({
  providedIn: 'root',
})
export class StatisticsService {
  // Private
  private __pagination = new BehaviorSubject<StatisticsPagination | undefined>(
    undefined
  );
  private __statistics = new BehaviorSubject<Statistics | undefined>(undefined);
  private __statisticss = new BehaviorSubject<Statistics[] | undefined>(
    undefined
  );

  /**
   * Constructor
   */
  constructor(private _httpClient: HttpClient) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Accessors
  // -----------------------------------------------------------------------------------------------------

  /**
   * Getter for pagination
   */
  get pagination$(): Observable<StatisticsPagination | undefined> {
    return this.__pagination.asObservable();
  }

  /**
   * Getter for statistics
   */
  get statistics$(): Observable<Statistics | undefined> {
    return this.__statistics.asObservable();
  }

  /**
   * Getter for statisticss
   */
  get statisticss$(): Observable<Statistics[] | undefined> {
    return this.__statisticss.asObservable();
  }

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Get statisticss
   *
   *
   * @param page
   * @param size
   * @param sort
   * @param order
   * @param search
   */
  getStatisticss(
    page: number = 0,
    size: number = 10,
    sort: string = 'name',
    order: 'asc' | 'desc' | '' = 'asc',
    search: string = ''
  ): Observable<{
    pagination: StatisticsPagination;
    statisticss: Statistics[];
  }> {
    return this._httpClient
      .get<{
        pagination: StatisticsPagination;
        statisticss: Statistics[];
      }>('api/apps/report/statistics/statisticss', {
        params: {
          page: '' + page,
          size: '' + size,
          sort,
          order,
          search,
        },
      })
      .pipe(
        tap((response) => {
          this.__pagination.next(response.pagination);
          this.__statisticss.next(response.statisticss);
        })
      );
  }

  /**
   * Get product by id
   */
  getStatisticsById(id: string | null): Observable<Statistics> {
    return this.__statisticss.pipe(
      take(1),
      map((statisticss) => {
        // Find the product
        const statistics =
          statisticss?.find((item) => item.id === id) || undefined;

        // Update the product
        this.__statistics.next(statistics);

        // Return the product
        return statistics;
      }),
      switchMap((product) => {
        if (!product) {
          return throwError('Could not found product with id of ' + id + '!');
        }

        return of(product);
      })
    );
  }

  /**
   * Create product
   */
  createStatistics(): Observable<Statistics> {
    return this.statisticss$.pipe(
      take(1),
      switchMap((statisticss) =>
        this._httpClient
          .post<Statistics>('api/apps/report/statistics/product', {})
          .pipe(
            map((newStatistics) => {
              // Update the statisticss with the new product
              if (!!statisticss) {
                this.__statisticss.next([newStatistics, ...statisticss]);
              }

              // Return the new product
              return newStatistics;
            })
          )
      )
    );
  }
}
